# Well...
# This is used to convert NoPHP -> PHP
# Highly experimental, be careful

from nophp.weaver import compiler

code = compiler.code 

print(code)

# The other way around:
# from phply.phplex import lexer
# from phply.phpparse import make_parser
# from phply import pythonast

# parser = make_parser()
# # body = [pythonast.from_phpast(ast)
# #         for ast in parser.parse(input.read(), lexer=lexer)]

# for ast in parser.parse(input(), lexer=lexer):
#     print(ast)


from ..exceptions import TranspilerExceptions, Warn
from ..module import Module
from ..types import *
from flask import session

class RedirectMod(Module):
    name="redirect"
    type = Module.MODULE_TYPES.FUNCTION

    def __init__(self, compiler_instance):
        super().__init__()
        self.compiler_instance = compiler_instance

    def base(self, tree, ref=False):
        values = []

        if 'FUNCTION_ARGUMENTS' not in tree:
            # Advanced handling
            for var in tree:
                value = self.ref_resolve(var) if ref else self.safely_resolve(var)
                values.append(value)
        else:
            if 'POSITIONAL_ARGS' in tree['FUNCTION_ARGUMENTS']:
                for var in tree['FUNCTION_ARGUMENTS']['POSITIONAL_ARGS']:

                    value = self.ref_resolve(var) if ref else self.safely_resolve(var)

                    values.append(value)

        return values

    def proc_tree(self, tree):
        # Dependencies
        resolution_module: Module = self.compiler_instance.get_action('RESOLUT')
        values = self.base(tree)


        if len(values) > 2:
            raise TranspilerExceptions.TooManyValues(values, "redirect($msg) or redirect($msg, $events[str])")
        

        if len(values) == 1:
            return String(f'<meta http-equiv="refresh" content="0;url={values[0]}">')
        elif len(values) == 2:
            # print(values[1])
            session['events'] = values[1]
            return String(f'<meta http-equiv="refresh" content="0;url={values[0]}">')
        else:
            raise TranspilerExceptions.Generic("Not enough values supplied to redirect")



from copy import deepcopy
from ..exceptions import TranspilerExceptions, Warn
from ..module import Module
from ..types import *


class CommonMod(Module):
    name="COMMON"
    type = Module.MODULE_TYPES.FUNCTION

    def __init__(self, compiler_instance):
        super().__init__()
        self.compiler_instance = compiler_instance

    def base(self, tree, ref=False):
        values = []

        if 'FUNCTION_ARGUMENTS' not in tree:
            # Advanced handling
            for var in tree:
                value = self.ref_resolve(var) if ref else self.safely_resolve(var)
                values.append(value)
        else:
            if 'POSITIONAL_ARGS' in tree['FUNCTION_ARGUMENTS']:
                for var in tree['FUNCTION_ARGUMENTS']['POSITIONAL_ARGS']:

                    value = self.ref_resolve(var) if ref else self.safely_resolve(var)

                    values.append(value)

        return values
    

# TODO: What is this for??? it's been a month since I wrote this??
# set_session($key, $value) -> <remembers value for the session>
class SetSessionMod(CommonMod):
    name="set_session"
    type = Module.MODULE_TYPES.FUNCTION

    def __init__(self, compiler_instance):
        super().__init__(compiler_instance)
        self.compiler_instance = compiler_instance

    def proc_tree(self, tree):
        values = self.base(tree)

        # TODO: Check if enough arguments are passed
        
        key = values[0]
        value = values[1]

        pass
    
    
_MODS = {
    "set_session": SetSessionMod,
}

def build_funcs(c):
    functions = {}
    for f in _MODS:
        functions[f] = {
                "run_func": _MODS[f](c)
            }
    return functions


from ..exceptions import TranspilerExceptions, Warn
from ..module import Module
from ..types import *
from importlib.metadata import version

class CommonInternalMod(Module):
    name="COMMONINTERNAL"
    type = Module.MODULE_TYPES.FUNCTION

    def __init__(self, compiler_instance):
        super().__init__()
        self.compiler_instance = compiler_instance

    def base(self, tree, ref=False):
        values = []

        if 'FUNCTION_ARGUMENTS' not in tree:
            # Advanced handling
            for var in tree:
                value = self.ref_resolve(var) if ref else self.safely_resolve(var)
                values.append(value)
        else:
            if 'POSITIONAL_ARGS' in tree['FUNCTION_ARGUMENTS']:
                for var in tree['FUNCTION_ARGUMENTS']['POSITIONAL_ARGS']:

                    value = self.ref_resolve(var) if ref else self.safely_resolve(var)

                    values.append(value)

        return values


# nophp_version() -> String($_version)
class VersionInfo(CommonInternalMod):
    name="nophp_version"
    type = Module.MODULE_TYPES.FUNCTION

    def __init__(self, compiler_instance):
        super().__init__(compiler_instance)
        self.compiler_instance = compiler_instance

    def proc_tree(self, tree):
        
        v = version("nophp")
        return String(v)
    
# die
class NoPHPDie(CommonInternalMod):
    name="die"
    type = Module.MODULE_TYPES.FUNCTION

    def __init__(self, compiler_instance):
        super().__init__(compiler_instance)
        self.compiler_instance = compiler_instance

    def proc_tree(self, tree):
        # Mirror functionality of INTERNAL_CALL(die)
        self.compiler_instance.stop = True


# typeof($var)
class NoPHPTypeOf(CommonInternalMod):
    name="typeof"
    type = Module.MODULE_TYPES.FUNCTION

    def __init__(self, compiler_instance):
        super().__init__(compiler_instance)
        self.compiler_instance = compiler_instance

    def proc_tree(self, tree):
        values = self.base(tree)
        value = values[0]

        return String(
            str(type(value))
        )
    

# toInt($var)
class NoPHPToInt(CommonInternalMod):
    name="toInt"
    type = Module.MODULE_TYPES.FUNCTION

    def __init__(self, compiler_instance):
        super().__init__(compiler_instance)
        self.compiler_instance = compiler_instance

    def proc_tree(self, tree):
        values = self.base(tree)
        value = values[0]

        return Int32(
            value
        )

class NoPHPTemplate(CommonInternalMod):
    name="template"
    type = Module.MODULE_TYPES.FUNCTION

    def __init__(self, compiler_instance):
        super().__init__(compiler_instance)
        self.compiler_instance = compiler_instance

    def proc_tree(self, tree):
        require = self.compiler_instance.get_action("REQUIRE_ONCE")
        require(tree)
        

_MODS = {
    "nophp_version": VersionInfo,
    "die": NoPHPDie,
    "typeof": NoPHPTypeOf,
    "toInt": NoPHPToInt,
    "template": NoPHPTemplate
}

def build_funcs(c):
    functions = {}
    for f in _MODS:
        functions[f] = {
                "run_func": _MODS[f](c)
            }
    return functions


from ..exceptions import TranspilerExceptions, Warn
from ..module import Module
from ..types import *

import sqlite3
sqlite3.enable_callback_tracebacks(True)

class DbCommonMod(Module):
    name="DBCOMMON"
    type = Module.MODULE_TYPES.FUNCTION

    def __init__(self, compiler_instance):
        super().__init__()
        self.compiler_instance = compiler_instance

    def base(self, tree):
        values = []

        if 'POSITIONAL_ARGS' in tree['FUNCTION_ARGUMENTS']:
            for var in tree['FUNCTION_ARGUMENTS']['POSITIONAL_ARGS']:

                value = self.safely_resolve(var)

                values.append(value)

        return values
    
    def safely_resolve(self, var, instance=None):
        if instance is None:
            instance = self.compiler_instance
        # Dependencies
        resolution_module: Module = self.compiler_instance.get_action('RESOLUT')
        func_module: Module = self.compiler_instance.get_action('FUNCTION_CALL')
        if type(var) == tuple:
            resolved = resolution_module(var)
        else: resolved = var
        value: BasicType = resolved

        if type(resolved) == Auto:
            value = self.safely_resolve(resolved.value)

        # print("\t\t", value)

        if type(value) == ID:
            value = value.value
            v = instance.get_variable(value)
            # print("var:",v)
            if v["type"] == String:
                value = self.remove_quotes(v['object'].value)
            elif v["type"] is None:
                value = ""
            elif v["type"] == Auto:
                value = self.safely_resolve(v["object"].value)
            elif v["type"] == ID:
                value = self.safely_resolve(v["object"], instance=instance.parent)
            elif type(v['object']) in BASE_TYPES:
                value = v['object']
            else:
                value = v['object'].value
        elif type(value) == String:
            value = self.remove_quotes(value.value)
            # print("\t", value)
        elif type(value) == Int32:
            value = value.value
        elif type(value) == sInnerMut:
            value = func_module.run_sInnerMut(value).value
        elif type(value) == SqlConnector:
            value = value.value
        elif type(value) == DynArray:
            _value = value.value
            value = []
            for i in _value:
                value.append(self.safely_resolve(i)) 
        elif type(value) == Float:
            value = value.value
        return value
    

# sql_connect("db.sql")
class DbConnect(DbCommonMod):
    name="DBCONN"
    type = Module.MODULE_TYPES.FUNCTION

    def __init__(self, compiler_instance):
        super().__init__(compiler_instance)
        self.compiler_instance = compiler_instance

    def proc_tree(self, tree):
        values = self.base(tree)

        connection = sqlite3.connect(values[0])

        return SqlConnector(connection)
    
# sql_query($conn, $sql)
class DbQuery(DbCommonMod):
    name="DBQ"
    type = Module.MODULE_TYPES.FUNCTION

    def __init__(self, compiler_instance):
        super().__init__(compiler_instance)
        self.compiler_instance = compiler_instance

    def proc_tree(self, tree):
        values = self.base(tree)


        if len(values) < 2:
            raise TranspilerExceptions.IncorrectNumberOfValues(values, "sql_query($conn, $sql)")
        
        conn, sql = values[:2]

        # print(values)

        returns = sql.split(' ')[0] == "SELECT"
            
        cursor: sqlite3.Cursor = conn.cursor()
        out = cursor.execute(sql)

        if returns:
            arr = DynArray([], type='auto')
            for val in out.fetchall():
                arr.value.append(
                    DynArray(list(val), type='auto')
                )
            return arr
        cursor.close()
        return Auto(out, type='basic')

# sql_query_one($conn, $sql)
class DbQueryOne(DbCommonMod):
    name="DBQ1"
    type = Module.MODULE_TYPES.FUNCTION

    def __init__(self, compiler_instance):
        super().__init__(compiler_instance)
        self.compiler_instance = compiler_instance

    def proc_tree(self, tree):
        values = self.base(tree)


        if len(values) < 2:
            raise TranspilerExceptions.IncorrectNumberOfValues(values, "sql_query_one($conn, $sql)")
        
        conn, sql = values[:2]

        # print(values)

        returns = sql.split(' ')[0] == "SELECT"
            
        cursor: sqlite3.Cursor = conn.cursor()
        out = cursor.execute(sql)

        if returns:
            return Auto(out.fetchone(), type='basic')
        cursor.close()
        return Auto(out)

# sql_close($conn)
class DbCloseConn(DbCommonMod):
    name="DBCLOSECONN"
    type = Module.MODULE_TYPES.FUNCTION

    def __init__(self, compiler_instance):
        super().__init__(compiler_instance)
        self.compiler_instance = compiler_instance

    def proc_tree(self, tree):
        values = self.base(tree)

        if len(values) < 1:
            raise TranspilerExceptions.IncorrectNumberOfValues(values, "sql_close($conn)")

        conn: sqlite3.Connection = values[0]
        conn.close()

# sql_query_sane($sql, $args[...])
class DbSane(DbCommonMod):
    name="DBSANE"
    type = Module.MODULE_TYPES.FUNCTION

    def __init__(self, compiler_instance):
        super().__init__(compiler_instance)
        self.compiler_instance = compiler_instance

    def proc_tree(self, tree):
        values =  self.base(tree)
        
        if len(values) < 3:
            raise TranspilerExceptions.IncorrectNumberOfValues(values, "sql_sane($conn, $sql, $args[...])")
        
        conn, sql, args = values[:3]

        returns = sql.split(' ')[0] == "SELECT"
            
        cursor: sqlite3.Cursor = conn.cursor()
        # print("Args:",args)
        out = cursor.execute(sql, args)


        # if returns:
        #     return Auto(out.fetchone(), type='basic')
        # cursor.close()
        # return Auto(out, type='basic')

        # print(out.fetchall())
    
        if returns:
            arr = DynArray([], type='auto')
            for val in out.fetchall():
                arr.value.append(
                    DynArray(list(val), type='basic')
                )
            # print(arr)
            return arr
        cursor.close()
        return Auto(out, type='basic')

#sql_commit_sane($conn, $sql, $args[...])
class DbSaneCommit(DbCommonMod):
    name="DBSANECOMMIT"
    type = Module.MODULE_TYPES.FUNCTION

    def __init__(self, compiler_instance):
        super().__init__(compiler_instance)
        self.compiler_instance = compiler_instance

    def proc_tree(self, tree):
        values =  self.base(tree)
        
        if len(values) < 3:
            raise TranspilerExceptions.IncorrectNumberOfValues(values, "sql_commit_sane($conn, $sql, $args[...])")
        
        conn, sql, args = values[:3]


        returns = sql.split(' ')[0] == "SELECT"
        if returns:
            raise TranspilerExceptions.Generic("sql_commit_sane only handles writes and updates, not select")
        
        try:
            cursor: sqlite3.Cursor = conn.cursor()
            cursor.execute(sql, args)
            out = cursor.lastrowid

            conn.commit()

            cursor.close()
            return Int32(out)
        except sqlite3.Error as e:
            print("DB Error:", e)
            return Nil(0)


_MODS = {
    "sql_connect": DbConnect,
    "sql_query": DbQuery,
    "sql_query_one": DbQueryOne,
    "sql_query_sane": DbSane,
    "sql_commit_sane": DbSaneCommit,
    "sql_close": DbCloseConn,

}

def build_funcs(c):
    functions = {}
    for f in _MODS:
        functions[f] = {
                "run_func": _MODS[f](c)
            }
    return functions


import random
from ..exceptions import TranspilerExceptions, Warn
from ..module import Module
from ..types import *


class RandMod(Module):
    name="rand"
    type = Module.MODULE_TYPES.FUNCTION

    def __init__(self, compiler_instance):
        super().__init__()
        self.compiler_instance = compiler_instance

    def proc_tree(self, tree):
        # Dependencies
        resolution_module: Module = self.compiler_instance.get_action('RESOLUT')
        values = []

        # Support weird ass calls
        if tree['FUNCTION_ARGUMENTS']['POSITIONAL_ARGS'][0][0] == "ARRAY_FILLED":
            tree['FUNCTION_ARGUMENTS']['POSITIONAL_ARGS'] = tree['FUNCTION_ARGUMENTS']['POSITIONAL_ARGS'][0][1]['ITEMS']

        for var in tree['FUNCTION_ARGUMENTS']['POSITIONAL_ARGS']:
            resolved = resolution_module(var)
            value: BasicType = None
            
            if type(resolved) == ID:
                value = resolved.value
                v = self.compiler_instance.get_variable(value)
                if v["type"] == Int32:
                    value = v['object'].value
                else:
                    raise TranspilerExceptions.TypeMissmatch(value, type(value), Int32, var[-1])
            elif type(resolved) == Int32:
                    value = resolved.value
            else:
                raise TranspilerExceptions.TypeMissmatch(resolved, type(resolved), Int32, var[-1])
            

            values.append(value) 

        if len(values) > 2:
            raise TranspilerExceptions.TooManyValues(values, "rand($min, $mix)")
        elif len(values) < 2:
            Warn("Too few values to satisfy 'rand'. Max will default to 2147483647")

        
        return Int32(random.randint(int(values[0]), int(values[1]) if len(values)>1 else 2147483647))



import stripe
from ..exceptions import TranspilerExceptions, Warn
from ..module import Module
from ..types import *

class CommonStripeMod(Module):
    name="COMMONSTRIPE"
    type = Module.MODULE_TYPES.FUNCTION

    def __init__(self, compiler_instance):
        super().__init__()
        self.compiler_instance = compiler_instance

    def base(self, tree, ref=False):
        values = []

        if 'FUNCTION_ARGUMENTS' not in tree:
            # Advanced handling
            for var in tree:
                value = self.ref_resolve(var) if ref else self.safely_resolve(var)
                values.append(value)
        else:
            if 'POSITIONAL_ARGS' in tree['FUNCTION_ARGUMENTS']:
                for var in tree['FUNCTION_ARGUMENTS']['POSITIONAL_ARGS']:

                    value = self.ref_resolve(var) if ref else self.safely_resolve(var)

                    values.append(value)

        return values


# stripe_set_key($value) -> Bool
class SetSecretMod(CommonStripeMod):
    name="stripe_set_key"
    type = Module.MODULE_TYPES.FUNCTION

    def __init__(self, compiler_instance):
        super().__init__(compiler_instance)
        self.compiler_instance = compiler_instance

    def proc_tree(self, tree):
        values = self.base(tree)

        key = values[0]

        stripe.api_key = key

        return Bool(True)
    
# stripe_intent($amount, $currency) -> String[client_secret]
class StripeIntentMod(CommonStripeMod):
    name="stripe_intent"
    type = Module.MODULE_TYPES.FUNCTION

    def __init__(self, compiler_instance):
        super().__init__(compiler_instance)
        self.compiler_instance = compiler_instance

    def proc_tree(self, tree):
        values = self.base(tree)

        amount, currency = values[0], values[1]
        intent = stripe.PaymentIntent.create(
            amount=amount,  
            currency=currency,
            # Additional options can be passed here (e.g., customer ID)
        )
        #print(amount, currency)

        return String(intent.client_secret)
    

# stripe_session($payment_types, $line_items, $success_url, $cancel_url) -> String[id]
class StripeNewSessionMod(CommonStripeMod):
    name="stripe_session"
    type = Module.MODULE_TYPES.FUNCTION

    def __init__(self, compiler_instance):
        super().__init__(compiler_instance)
        self.compiler_instance = compiler_instance

    def proc_tree(self, tree):
        values = self.base(tree)

        payment_types, line_items, success, cancel = values[0], values[1], values[2], values[3]

        try:
            session = stripe.checkout.Session.create(
                payment_method_types=payment_types,
                line_items=line_items,
                # [{
                #     'price_data': {
                #         'currency': 'usd',
                #         'product_data': {
                #             'name': 'T-shirt',
                #         },
                #         'unit_amount': 2000,
                #     },
                #     'quantity': 1,
                # }],
                mode='payment',
                success_url=success,
                cancel_url=cancel,
            )
            return String(session.id)
        except Exception as e:
            return String(f"Failed due to {e}")

    
_MODS = {
    "stripe_set_key": SetSecretMod,
    "stripe_intent": StripeIntentMod,
    "stripe_session": StripeNewSessionMod,
}

def build_funcs(c):
    functions = {}
    for f in _MODS:
        functions[f] = {
                "run_func": _MODS[f](c)
            }
    return functions


from sly import Lexer

class PyettyLexer(Lexer):
    tokens = {
        ID,
        FLOAT,
        INT,
        CHAR,
        FUNC,
        CLASS,
        NAMESPACE,
        STRING,
        MULTILINE_STRING,
        FORMATTED_STRING,
        EQ_GREATER,
        EQ_LESS,
        EQEQ,
        PYTHON_CODE,
        COLON_COLON,
        IF,
        ELSE,
        TRUE,
        FALSE,
        NOT_EQEQ,
        WHILE,
        BREAK,
        SKIP,
        FOR,
        FOREACH,
        AS,
        IN,
        DEL,
        RETURN,
        NULL,
        EQ_ADD,
        EQ_SUB,
        EQ_MUL,
        EQ_DIV,
        EQ_MOD,
        IMPORT,
        LIMPORT,
        SANDBOX,
        FARROW,
        TARROW,
        LET,
        TELSE,
        PYTHON_CODE_EXEC,
        OF,
        GLOBAL,
        DEFINE,
        DEBUG,
        DEPENDS,
        SELFISH,
        ENV,
        FROM,
        INIT,
        NEW,
        PHPSTART,
        PHPEND,
        HTMLSTART,
        # HTMLSTRING,
        HTMLEND,
        SARROW,
        # TEXT
        EXTENDS,
        ATTR_LEVEL,
        USING,
    }
    literals = {
        "+",
        "-",
        "*",
        "/",
        "%",
        "|",
        "&",
        "!",
        ">",
        "<",
        "=",
        "(",
        ")",
        "{",
        "}",
        ";",
        ",",
        ":",
        "[",
        "]",
        "\\",
        "/",
        ".",
        "?",
        "^",
        "#",
        "_",
        "$"
    }

    ignore = " \t"
    ignore_comment_slash = r"//.*"

    FLOAT = r"\d*\.\d+"
    INT = r"\d+"

    PYTHON_CODE = r"\$`[.\W\w]*?`"
    PYTHON_CODE_EXEC = r"\$e`[.\W\w]*?`"
    STRING = r"(\".*?(?<!\\)(\\\\)*\"|'.*?(?<!\\)(\\\\)*')" #r'"[\s\S]*?"' 
    MULTILINE_STRING = r"(`(?:[^`\\]|\\.)*`)"
    FORMATTED_STRING = r"f(`(?:[^`\\]|\\.)*`)"
    CHAR = r"'[\s\S]*?'"
    ID = r"(--[a-zA-Z_]([a-zA-Z0-9_]|!)*--|[a-zA-Z_]([a-zA-Z0-9_]|!)*)"
    PHPSTART = r"<\?php"
    PHPEND = r"\?>"
    HTMLEND = r'</([^<>]+)>'
    HTMLSTART = r'<([^<>]+)>' #r"(\".*?(?<!\\)(\\\\)*\"|'.*?(?<!\\)(\\\\)*')"
    ID["function"] = FUNC
    ID["class"] = CLASS
    ID["extends"] = EXTENDS
    ID["namespace"] = NAMESPACE
    ID["break"] = BREAK
    ID["skip"] = SKIP
    ID["true"] = TRUE
    ID["false"] = FALSE
    ID["while"] = WHILE
    ID["for"] = FOR
    ID["foreach"] = FOREACH
    ID["in"] = IN
    ID["as"] = AS
    ID["if"] = IF
    ID["else"] = ELSE
    ID["del"] = DEL
    ID["null"] = NULL
    ID["return"] = RETURN
    ID["import"] = IMPORT
    ID["limport"] = LIMPORT
    ID["sandbox"] = SANDBOX
    ID["let"] = LET
    ID["of"] = OF
    ID["globals"] = GLOBAL
    ID["define"] = DEFINE
    ID["depends"] = DEPENDS
    ID["debugThis"] = DEBUG
    ID["selfish"] = SELFISH
    ID["env"] = ENV
    ID["from"] = FROM
    ID["_init"] = INIT
    ID["new"] = NEW
    ID["using"] = USING
    ATTR_LEVEL = r'(public|private|protected)'
    # TEXT = r'>\s*([^<>\s]+)\s*'
    # HTMLSTRING = r'[\s\S]+?'

    TARROW = r'->' # Tiny arrow
    FARROW = r'\=\=>' # Fat arrow
    SARROW = r'\=\>' # Skinny arrow
    TELSE = r'\|\|'
    COLON_COLON = r"::"
    EQEQ = "=="
    NOT_EQEQ = r"!="
    EQ_GREATER = r"=>"
    EQ_LESS = r"=<"
    EQ_ADD = r"\+="
    EQ_SUB = r"-="
    EQ_MUL = r"\*="
    EQ_DIV = r"/="
    EQ_MOD = r"%="

    

    @_(r"\n+")
    def ignore_newline(self, t):
        self.lineno += len(t.value)



from ..exceptions import TranspilerExceptions, Warn
from ..module import Module
from ..types import *

# Courtesy of AlejandroVD at StackOverflow
def htmlspecialchars(text):
    return (
        text.replace("&", "&amp;").
        replace('"', "&quot;").
        replace("<", "&lt;").
        replace(">", "&gt;")
    )
def invert_htmlspecialchars(text):
    return (
        text.replace( "&amp;", "&").
        replace("&quot;", '"').
        replace("&lt;", "<").
        replace("&gt;", ">")
    )


class HTMLSpecialCharMod(Module):
    name="htmlspecialchars"
    type = Module.MODULE_TYPES.FUNCTION

    def __init__(self, compiler_instance):
        super().__init__()
        self.compiler_instance = compiler_instance

    def proc_tree(self, tree):
        # Dependencies
        resolution_module: Module = self.compiler_instance.get_action('RESOLUT')
        values = []

        for var in tree['FUNCTION_ARGUMENTS']['POSITIONAL_ARGS']:
            resolved = resolution_module(var)
            value: BasicType = None
            
            if type(resolved) == ID:
                value = resolved.value
                v = self.compiler_instance.get_variable(value)
                if v["type"] == String:
                    value = self.remove_quotes(v['object'].value)
                elif v["type"] == type(None):
                    value = ""
                else:
                    value = v['object'].value
            elif type(resolved) == String:
                value = self.remove_quotes(resolved.value)

            

            values.append(value) 

        if len(values) > 1:
            raise TranspilerExceptions.TooManyValues(values, "htmlspecialchars($msg)")
        


        return String(htmlspecialchars(str(value)))

class InvertHTMLSpecialCharMod(Module):
    name="inverthtmlspecialchars"
    type = Module.MODULE_TYPES.FUNCTION

    def __init__(self, compiler_instance):
        super().__init__()
        self.compiler_instance = compiler_instance

    def proc_tree(self, tree):
        # Dependencies
        resolution_module: Module = self.compiler_instance.get_action('RESOLUT')
        values = []

        for var in tree['FUNCTION_ARGUMENTS']['POSITIONAL_ARGS']:
            resolved = resolution_module(var)
            value: BasicType = None
            
            if type(resolved) == ID:
                value = resolved.value
                v = self.compiler_instance.get_variable(value)
                if v["type"] == String:
                    value = self.remove_quotes(v['object'].value)
                elif v["type"] == type(None):
                    value = ""
                else:
                    value = v['object'].value
            elif type(resolved) == String:
                value = self.remove_quotes(resolved.value)

            

            values.append(value) 

        if len(values) > 1:
            raise TranspilerExceptions.TooManyValues(values, "invert_htmlspecialchars($msg)")
        


        return String(invert_htmlspecialchars(str(value)))
